using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using SnakeGame.Common.Objects;
using SnakeGame.Common.Interfaces;

namespace SnakeApp
{
    public delegate void OnFinished();

    public class Game : GenericGame
    {
        

        protected Panel[][] displayGrid;
        protected Label scoreDisplay;
        protected Label levelDisplay;
        public event OnFinished OnFinished;

        public Game(Panel[][] pnls, Label scr, Label lvl, int x, int y) : base(x,y)
        {
            displayGrid = pnls;
            scoreDisplay = scr;
            levelDisplay = lvl;
            
            for (int i = 0; i < width; i++)
            {
                for(int j=0; j<height; j++)
                {
                    switch (statusGrid[i][j])
                    {
                        case LocationStatus.APPLE:
                            displayGrid[i][j].BackColor = Color.Red;
                            break;
                        case LocationStatus.WALL:
                            displayGrid[i][j].BackColor = Color.Blue;
                            break;
                        case LocationStatus.SNAKE:
                            displayGrid[i][j].BackColor = Color.Green;
                            break;
                        case LocationStatus.EMPTY:
                            displayGrid[i][j].BackColor = Color.Black;
                            break;
                    }
                }
            }
        }

        override public void add(int x, int y)
        {
            base.add(x, y);
            displayGrid[x][y].BackColor = Color.Green;
        }

        override public void remove(int x, int y)
        {
            base.remove(x, y);
            displayGrid[x][y].BackColor = Color.Black;
        }

        public void run(object CallbackObject)
        {
            snake.init();
            while (!snake.dead)
            {
                System.Threading.Thread.Sleep(100);
                try
                {
                    ((Form)(CallbackObject)).Invoke(
                    new MethodInvoker(delegate { step(); }));
                }
                catch (Exception e)
                {
                    if(e!=null) return;
                }
            }
            OnFinished();
        }

        public override void step()
        {
            base.step();
            for (int i = 0; i < width; i++)
            {
                for (int j = 0; j < height; j++)
                {
                    switch (statusGrid[i][j])
                    {
                        case LocationStatus.APPLE:
                            displayGrid[i][j].BackColor = Color.Red;
                            break;
                        case LocationStatus.WALL:
                            displayGrid[i][j].BackColor = Color.Blue;
                            break;
                        case LocationStatus.SNAKE:
                            displayGrid[i][j].BackColor = Color.Green;
                            break;
                        case LocationStatus.EMPTY:
                            displayGrid[i][j].BackColor = Color.Black;
                            break;
                    }
                }
                if (score != Convert.ToInt32(scoreDisplay.Text))
                {
                    scoreDisplay.Text = score.ToString();
                }
                if (level.ToString() != levelDisplay.Text)
                {
                    levelDisplay.Text = level.ToString();
                }
            }
        }
    }
}
