using System;
using System.Collections.Generic;
using System.Text;
using SnakeGame.Common.Objects;
using SnakeGame.Common.Interfaces;

namespace SnakeGame.Common.Objects
{
    public delegate void OnFinished();

    public class GenericGame : IGame
    {
        protected LocationStatus[][] statusGrid;
        public bool gameover = false;
        protected Snake snake;
        protected int width;
        protected int height;
        protected Random rand;
        protected int appleDelay;
        protected bool first;
        protected ILevel level;
        protected int score;
        protected IGame passThruReference;

        public void passThru(IGame refr){
            snake.bindGame(refr);
            passThruReference = refr;
        }

        public GenericGame(int x, int y)
        {
            passThruReference = this;
            score = 0;
            level = new Level01();
            width = x;
            height = y;
            first = true;
            statusGrid=new LocationStatus [width][];
            for (int i = 0; i < width; i++)
            {
                statusGrid[i] = new LocationStatus[height];
                for(int j=0; j<height; j++)
                {
                    statusGrid[i][j] = LocationStatus.EMPTY;
                }
            }
            rand = new Random();
            for (int i = 0; i < 6; i++)
            {
                x=rand.Next(width-1);
                y=rand.Next(height-1);
                statusGrid[x][y]=LocationStatus.APPLE;
            }
            level.getPlayField(statusGrid, width, height);
            /*
            for (int i = 4; i < width - 4; i++)
            {
                int j = height / 2;
                statusGrid[i][j] = LocationStatus.WALL;
            }
            for (int i = 0; i < width; i++)
            {
                statusGrid[i][0] = LocationStatus.WALL;
                statusGrid[i][height - 1] = LocationStatus.WALL;
            }
            for (int j = 0; j < height; j++)
            {
                statusGrid[0][j] = LocationStatus.WALL;
                statusGrid[width - 1][j] = LocationStatus.WALL;
            } 
            */
            appleDelay = 7+rand.Next(8);
            snake = new Snake(this);
        }

        public virtual void left() { if (snake.direction != Snake.Direction.RIGHT) snake.direction = Snake.Direction.LEFT; }
        public virtual void right() { if (snake.direction != Snake.Direction.LEFT ) snake.direction = Snake.Direction.RIGHT; }
        public virtual void up() { if(snake.direction!=Snake.Direction.DOWN) snake.direction=Snake.Direction.UP; }
        public virtual void down() { if (snake.direction != Snake.Direction.UP) snake.direction = Snake.Direction.DOWN; }

        public virtual LocationStatus getStatus(int x, int y)
        {
            if (x < 0 || x >= width || y < 0 || y >= height)
            {
                return LocationStatus.WALL;
            }
            return statusGrid[x][y];
        }

        public virtual void add(int x, int y)
        {
            statusGrid[x][y] = LocationStatus.SNAKE;
        }

        public virtual void remove(int x, int y)
        {
            statusGrid[x][y] = LocationStatus.EMPTY;
        }

        public void init()
        {
            snake.init();
        }

        public virtual void step()
        {
            snake.step();
            appleDelay--;
            if (appleDelay <= 0)
            {
                appleDelay = 7+rand.Next(8);
                int x = rand.Next(width - 1);
                int y = rand.Next(height - 1);
                if (statusGrid[x][y] == LocationStatus.EMPTY)
                {
                    statusGrid[x][y] = LocationStatus.APPLE;
                }
            }
            if (snake.dead) gameover = true;
        }

        public void clearGrid()
        {
            for (int i = 0; i < width; i++)
            {
                for (int j = 0; j < height; j++)
                {
                    statusGrid[i][j] = LocationStatus.EMPTY;
                }
            }
        }

        public virtual void eat()
        {
            score += 100;
            if (score == 2000)
            {
                loadLevel(new Level02());
                throw new ApplicationException();
            }
            if (score == 4000)
            {
                loadLevel(new Level03());
                throw new ApplicationException();
            }
        }
        public virtual void loadLevel(ILevel l)
        {
            clearGrid();
            level = l;
            level.getPlayField(statusGrid, width, height);
            snake.init();
        }
    }
}
