using System;
using System.Collections.Generic;
using System.Text;
using SnakeGame.Common.Interfaces;
using SnakeGame.Common.Objects;

namespace SnakeGame.Common.Objects
{
    public class Snake
    {
        public enum Direction
        {
            UP,
            DOWN,
            LEFT,
            RIGHT
        }

        public LinkedList<Segment> segments = new LinkedList<Segment>();
        public Direction direction;
        public bool dead;
        int snakeLength;
        IGame game;

        public Snake(IGame parent)
        {
            
            game = parent;
            direction = Direction.RIGHT;
            dead = false;
            snakeLength = 10;
        }

        public Snake(Segment start, IGame parent) : this(parent)
        {
            //segments.AddFirst(new LinkedListNode<Segment>(start));
        }

        public void bindGame(IGame refr)
        {
            game = refr;
        }

        public void init()
        {
            segments.Clear();
            segments.AddFirst(new LinkedListNode<Segment>(new Segment(1, 1)));
            game.add(segments.First.Value.X, segments.First.Value.Y);
            direction = Direction.RIGHT;
            snakeLength = 10;
        }

        public void step()
        {
            Segment head = segments.First.Value;
            int nextx = head.X;
            int nexty = head.Y;
            switch (direction)
            {
                case Direction.UP:
                    nexty -= 1;
                    break;
                case Direction.DOWN:
                    nexty += 1;
                    break;
                case Direction.LEFT:
                    nextx -= 1;
                    break;
                case Direction.RIGHT:
                    nextx += 1;
                    break;
            }
            Segment nextLink = new Segment(nextx, nexty);
            segments.AddFirst(new LinkedListNode<Segment>(nextLink));
            if ((game.getStatus(nextx, nexty) != LocationStatus.EMPTY)&&
                (game.getStatus(nextx, nexty) != LocationStatus.APPLE))
            {
                dead = true;
                return;
            }
            try
            {
                if (game.getStatus(nextx, nexty) == LocationStatus.APPLE)
                {
                    snakeLength += 3;
                    game.eat();
                }

                game.add(nextx, nexty);
                while (segments.Count > snakeLength)
                {
                    Segment old = segments.Last.Value;
                    game.remove(old.X, old.Y);
                    segments.RemoveLast();
                }
            }
            catch (Exception e)
            {
                if (e != null) return;
            }
            
        }
    }
}
